@@ -33,6 +33,7 @@ from twarc.decorators2 import (
     cli_api_error,
     TimestampProgressBar,
     FileSizeProgressBar,
+    FileLineProgressBar,
     _millis2snowflake,
     _date2millis,
 )
@@ -614,7 +615,7 @@ def hydrate(T, infile, outfile, hide_progress):
     """
     Hydrate tweet ids.
     """
-    with FileSizeProgressBar(infile, outfile, disable=hide_progress) as progress:
+    with FileLineProgressBar(infile, outfile, disable=hide_progress) as progress:
         for result in T.tweet_lookup(infile):
             _write(result, outfile)
             tweet_ids = [t["id"] for t in result.get("data", [])]
@@ -705,7 +706,7 @@ def users(T, infile, outfile, usernames, hide_progress):
     """
     Get data for user ids or usernames.
     """
-    with FileSizeProgressBar(infile, outfile, disable=hide_progress) as progress:
+    with FileLineProgressBar(infile, outfile, disable=hide_progress) as progress:
         for result in T.user_lookup(infile, usernames):
             _write(result, outfile)
             if usernames:
@@ -952,9 +953,9 @@ def timelines(
     line_count = 0
     seen = set()
 
-    with FileSizeProgressBar(infile, outfile, disable=hide_progress) as progress:
+    with FileLineProgressBar(infile, outfile, disable=hide_progress) as progress:
         for line in infile:
-            progress.update(len(line))
+            progress.update()
             line_count += 1
             line = line.strip()
             if line == "":
@@ -1067,6 +1068,234 @@ def _timeline_tweets(
     return tweets
 
 
+@twarc2.command("searches")
+@click.option("--since-id", type=int, help="Match tweets sent after tweet id")
+@click.option("--until-id", type=int, help="Match tweets sent prior to tweet id")
+@click.option(
+    "--start-time",
+    type=click.DateTime(formats=("%Y-%m-%d", "%Y-%m-%dT%H:%M:%S")),
+    help="Match tweets created after UTC time (ISO 8601/RFC 3339), e.g.  2021-01-01T12:31:04",
+)
+@click.option(
+    "--end-time",
+    type=click.DateTime(formats=("%Y-%m-%d", "%Y-%m-%dT%H:%M:%S")),
+    help="Match tweets sent before UTC time (ISO 8601/RFC 3339)",
+)
+@click.option(
+    "--archive",
+    is_flag=True,
+    default=False,
+    help="Search the full archive (requires Academic Research track). Defaults to searching the entire twitter archive if --start-time is not specified.",
+)
+@click.option(
+    "--limit",
+    default=0,
+    help="Maximum number of tweets to save *per search*, ignored if --counts-only is specified.",
+)
+@click.option(
+    "--hide-progress",
+    is_flag=True,
+    default=False,
+    help="Hide the Progress bar. Default: show progress, unless using pipes.",
+)
+@click.option(
+    "--counts-only",
+    is_flag=True,
+    default=False,
+    help="Only retrieve counts of tweets matching the search, not the tweets themselves. "
+    "outfile will be a CSV containing the counts for all of the queries in the input file.",
+)
+@click.option(
+    "--combine-queries",
+    is_flag=True,
+    default=False,
+    help="""Merge consecutive queries into a single OR query.
+    For example, if the three rows in your file are: banana, apple, pear
+    then a single query ((banana) OR (apple) OR (pear)) will be issued.
+    """,
+)
+@click.option(
+    "--granularity",
+    default="day",
+    type=click.Choice(["day", "hour", "minute"], case_sensitive=False),
+    help="Aggregation level for counts (only used when --count-only is used). Can be one of: day, hour, minute. Default is day.",
+)
+@click.argument("infile", type=click.File("r"), default="-")
+@click.argument("outfile", type=click.File("w"), default="-")
+@click.pass_obj
+def searches(
+    T,
+    infile,
+    outfile,
+    limit,
+    since_id,
+    until_id,
+    start_time,
+    end_time,
+    archive,
+    counts_only,
+    granularity,
+    combine_queries,
+    hide_progress,
+):
+    """
+    Execute each search in the input file, one at a time.
+
+    The infile must be a file containing one query per line. Each line will be
+    passed through directly to the Twitter API - unlike the timelines command
+    quotes will not be removed.
+
+    Input queries will be deduplicated - if the same literal query is present
+    in the file, it will still only be run once.
+
+    It is recommended that this command first be run with --counts-only, to
+    check that each of the queries is retrieving the volume of tweets
+    expected, and to avoid consuming quota unnecessarily.
+
+    """
+    line_count = 0
+    seen = set()
+
+    # Make sure times are always in UTC, click sometimes doesn't add timezone:
+    if start_time is not None and start_time.tzinfo is None:
+        start_time = start_time.replace(tzinfo=timezone.utc)
+    if end_time is not None and end_time.tzinfo is None:
+        end_time = end_time.replace(tzinfo=timezone.utc)
+
+    # Standard API max query length
+    max_query_length = 512
+
+    # TODO: this duplicates existing logic in _search, but _search is too
+    # specific to be reused here.
+    if archive:
+        # start time defaults to the beginning of Twitter to override the
+        # default of the last month. Only do this if start_time is not already
+        # specified and since_id and until_id aren't being used
+        if start_time is None and since_id is None and until_id is None:
+            start_time = datetime.datetime(2006, 3, 21, tzinfo=datetime.timezone.utc)
+
+        # Academic track let's you use longer queries
+        max_query_length = 1024
+
+    if counts_only:
+        api_method = T.counts_all if archive else T.counts_recent
+        api_data = (
+            since_id,
+            until_id,
+            start_time,
+            end_time,
+            granularity,
+        )
+
+        # Write the header for the CSV output
+        click.echo(f"query,start,end,{granularity}_count", file=outfile)
+
+    else:
+        api_method = T.search_all if archive else T.search_recent
+        api_data = (
+            since_id,
+            until_id,
+            start_time,
+            end_time,
+        )
+
+    # TODO: Validate the queries are all valid length before beginning and report errors
+
+    # TODO: Needs an inputlines progress bar instead, as the queries are variable
+    # size.
+    with FileLineProgressBar(infile, outfile, disable=hide_progress) as progress:
+
+        merged_query = ""
+
+        for query in infile:
+            query = query.strip()
+
+            progress.update(1)
+            line_count += 1
+
+            if query == "":
+                log.warn("skipping blank line on line %s", line_count)
+                continue
+
+            if len(query) >= max_query_length:
+                log.warn(f"skipping too long query {query} on line {line_count}")
+                continue
+
+            if query in seen:
+                log.info("already processed %s, skipping", query)
+                continue
+
+            seen.add(query)
+            retrieved = 0
+
+            if combine_queries and merged_query:
+                extended_query = f"{merged_query} OR ({query})"
+                # We've exceeded the limit, so now we can issue
+                # the merged query.
+                if len(extended_query) >= max_query_length:
+                    issue_query = merged_query
+                    merged_query = f"({query})"
+                else:
+                    # We haven't exceed the length yet, so accept the addon
+                    merged_query = extended_query
+                    continue
+
+            elif combine_queries:
+                merged_query = f"({query})"
+                continue
+
+            else:
+                # This is the normal case - we are not doing any combination.
+                issue_query = query
+
+            log.info(f'Beginning search for "{issue_query}"')
+
+            response = api_method(issue_query, *api_data)
+
+            for result in response:
+
+                if counts_only:
+                    for r in result["data"]:
+                        click.echo(
+                            f'{issue_query},{r["start"]},{r["end"]},{r["tweet_count"]}',
+                            file=outfile,
+                        )
+
+                else:
+                    # Apply the limit if not counting
+                    _write(result, outfile)
+
+                    retrieved += len(result["data"])
+
+                    if limit and (retrieved >= limit):
+                        break
+
+        # Make sure to process the final batch of queries if using the combined strategy
+        if combine_queries and (
+            merged_query == extended_query or merged_query == f"({query})"
+        ):
+            log.info(f'Beginning search for "{merged_query}"')
+            response = api_method(merged_query, *api_data)
+
+            for result in response:
+
+                if counts_only:
+                    for r in result["data"]:
+                        click.echo(
+                            f'{merged_query},{r["start"]},{r["end"]},{r["tweet_count"]}',
+                            file=outfile,
+                        )
+
+                else:
+                    # Apply the limit if not counting
+                    _write(result, outfile)
+
+                    retrieved += len(result["data"])
+
+                    if limit and (retrieved >= limit):
+                        break
+
+
 @twarc2.command("conversation")
 @click.option("--since-id", type=int, help="Match tweets sent after tweet id")
 @click.option("--until-id", type=int, help="Match tweets sent prior to tweet id")
@@ -1173,9 +1402,9 @@ def conversations(
     count = 0
     stop = False
 
-    with FileSizeProgressBar(infile, outfile, disable=hide_progress) as progress:
+    with FileLineProgressBar(infile, outfile, disable=hide_progress) as progress:
         for line in infile:
-            progress.update(len(line))
+            progress.update()
             conv_ids = []
 
             # stop will get set when the total tweet limit has been met
